"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
}
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
}
Object.defineProperty(exports, "__esModule", { value: true });
const koa_compose_1 = __importDefault(require("koa-compose"));
const WebSocket = __importStar(require("ws"));
const base64id_1 = __importDefault(require("base64id"));
const socket_1 = __importDefault(require("./socket"));
class WsSocket {
    constructor() {
        this.middleware = [];
        this.composed = koa_compose_1.default([]);
        this.connections = new Map();
        this.listeners = new Map();
        this.wss = null;
        this.onConnection = (ws, request) => {
            const socketId = base64id_1.default.generateId();
            const socketInstance = new socket_1.default(ws, request, this.listeners, this.composed);
            this.connections.set(socketId, socketInstance);
            ws.on('close', () => {
                this.connections.delete(socketId);
            });
            const handlers = this.listeners.get('connection');
            if (handlers) {
                handlers.forEach(handler => {
                    handler({
                        event: 'connection',
                        data: socketInstance,
                        request,
                        socket: socketInstance.ws,
                    }, null);
                });
            }
        };
    }
    attach(app, https = false, tlsOpts = {}) {
        const enhancedApp = app;
        if (enhancedApp.server &&
            enhancedApp.server.constructor.name !== 'Server') {
            throw new Error("app.server already exists but it's not an http server");
        }
        if (!enhancedApp.server) {
            // Create a server if it doesn't already exists
            enhancedApp.server = https
                ? require('https').createServer(tlsOpts, enhancedApp.callback())
                : require('http').createServer(enhancedApp.callback());
            // Patch `app.listen()` to call `app.server.listen()`
            enhancedApp.listen = function listen() {
                enhancedApp.server.listen.apply(enhancedApp.server, arguments);
                return enhancedApp.server;
            };
        }
        this.wss = new WebSocket.Server({ server: enhancedApp.server });
        enhancedApp.ws = this;
        this.wss.on('connection', this.onConnection);
    }
    /**
     * Pushes a middleware on to the stack
     * @param fn <Function> the middleware function to execute
     */
    use(fn) {
        this.middleware.push(fn);
        this.composed = koa_compose_1.default(this.middleware);
        this.updateConnections();
        return this;
    }
    /**
     * Adds a new listener to the stack
     * @param event <String> the event id
     * @param handler <Function> the callback to execute
     * @return this
     */
    on(event, handler) {
        const listeners = this.listeners.get(event) || [];
        listeners.push(handler);
        this.listeners.set(event, listeners);
        this.updateConnections();
        return this;
    }
    /**
     * Removes a listener from the event
     * @param event <String> if omitted will remove all listeners
     * @param handler <Function> if omitted will remove all from the event
     * @return this
     */
    off(event, handler) {
        if (!event) {
            this.listeners = new Map();
            this.updateConnections();
            return this;
        }
        if (!handler) {
            this.listeners.delete(event);
            this.updateConnections();
            return this;
        }
        const listeners = this.listeners.get(event) || [];
        let i = listeners.length - 1;
        while (i) {
            if (listeners[i] === handler) {
                break;
            }
            i--;
        }
        listeners.splice(i, 1);
        this.updateConnections();
        return this;
    }
    /**
     * Updates all existing connections with current listeners and middleware
     * @private
     */
    updateConnections() {
        this.connections.forEach(connection => connection.update(this.listeners, this.composed));
    }
}
exports.WsSocket = WsSocket;
//# sourceMappingURL=index.js.map